import os
# import pandas as pd


class CastorClient(object):

    def __init__(self):
        self.current_dir = os.path.abspath(os.path.curdir)
        # self.data = None

    def print_current_dir(self):
        print(self.current_dir)

    # def pwd(self):
    #     return self.current_dir
    #
    # def cd(self, d):
    #     self.current_dir = os.path.join(self.current_dir, d)
    #     self.pwd()
    #
    # def ls(self):
    #     entries = []
    #     for f in os.listdir(self.current_dir):
    #         entries.append(f)
    #     return entries
    #
    # def load_excel(self, file_name):
    #     if os.path.isfile(file_name):
    #         file_path = file_name
    #     else:
    #         file_path = os.path.join(self.current_dir, file_name)
    #     self.data = pd.read_excel(file_path)
    #     return self.data



import os
import pydicom
from pydicom._dicom_dict import DicomDictionary
from barbell2.lib import BasicShell


INTRO = """
Welcome to the DICOM Explorer!
------------------------------
This tool allows you to explore lots of DICOM files at the same time. Type 'help' to view a short list of
commands. Type 'help <command>' to view details about a given command.
"""

PROMPT = '(shell) '


def is_dicom(file_path):
    if not os.path.isfile(file_path):
        return False
    try:
        with open(file_path, "rb") as f:
            return f.read(132).decode("ASCII")[-4:] == "DICM"
    except:
        return False


def get_institute_name(file_path):
    items = os.path.split(file_path)
    institute_name = os.path.split(items[0])[1]
    return institute_name


class DicomExplorerShell(BasicShell):

    def __init__(self):
        super(DicomExplorerShell, self).__init__()
        self.intro = INTRO
        self.prompt = PROMPT
        self.debug = True

    # LOADING FILES AND DIRECTORIES

    def do_load_file(self, file_path):
        """
        Usage: load_file <file name or path>
        Load a single DICOM file. If only the name is provided, it assumed that the file is located in the
        current directory.
        """
        if not os.path.isfile(file_path):
            file_path = os.path.join(self.current_dir, file_path)
        if not is_dicom(file_path):
            self.poutput('File is not DICOM')
            return
        self.poutput('Loading...')
        self.result_manager.add_result_data([file_path])
        self.poutput('Loading done')

    def do_load_dir(self, dir_path):
        """
        Usage: load_dir <dir name or path>
        Load (recursively) all DICOM files in the given directory. If only the directory name is given, it is
        assumed that the directory is located in the current directory.
        """
        if not os.path.isdir(dir_path):
            dir_path = os.path.join(self.current_dir, dir_path)
        self.poutput('Loading {}...'.format(dir_path))
        data = []
        for root, dirs, files in os.walk(dir_path):
            for f in files:
                if not f.startswith('._'):
                    f = os.path.join(root, f)
                    if is_dicom(f):
                        data.append(f)
                        self.poutput(f)
        self.result_manager.add_result_data(data)
        self.poutput('Loading done')

    def do_show_files(self, n):
        """
        Usage: show_files [n]
        Show all (or first n) files loaded in the current result set. If you want to show files from another result set
        select it first using the set_current_result command.
        """
        files = self.result_manager.get_current_result_data()
        n = -1 if n == '' else int(n)
        count = 0
        for f in files:
            self.poutput(f)
            if count == n:
                break
            count += 1
        self.poutput('Total nr. of files: {}'.format(len(files)))

    def do_lookup(self, tag_name):
        """
        Usage: lookup <tag name>
        Lookup tag <tag name> in the DICOM dictionary. You can specify only parts of a tag name, e.g.,
        "Transmit" will return multiple dictionary entries containing the word "Transmit" (like Transmit
        Coil Name). Being able to lookup tags comes in handy when you search for them in DICOM files.
        Note that (parts of) the tag name does not have to be case-sensitive.
        """
        for key, value in DicomDictionary.items():
            output = '{}: {}'.format(key, value)
            if tag_name == '':
                self.poutput(output)
            else:
                for item in value:
                    if tag_name in item:
                        self.poutput(output)

    def do_find_tag(self, tag_name):
        """
        Usage: find_tag <tag name>
        Find tag <tag name> in the currently loaded DICOM files.
        """
        files = self.result_manager.get_current_result_data()
        for f in files:
            p = pydicom.read_file(f)
            for tag in p.keys():
                if tag in list(DicomDictionary.keys()):
                    if tag_name == DicomDictionary[tag][4]:
                        self.poutput('{}: {}'.format(f, p[tag].value))
                        break
                else:
                    self.poutput('Warning: tag {} cannot be found in the DICOM dictionary'.format(tag))

    def do_show_header(self, file_name):
        """
        Usage: show_header <file name>
        Show DICOM header of <file name>. If <file name> is left empty, this function will search for
        all file paths that contain the given file name. If only one is encountered, the DICOM header will
        be displayed. If multiple files are encountered, a list of those files is displayed so that you can
        select a specific one (with its full path).
        """
        files = self.result_manager.get_current_result_data()
        count = 0
        counted_files = []
        for f in files:
            if file_name in f:
                count += 1
                counted_files.append(f)
        if count == 0:
            if os.path.isfile(file_name):
                p = pydicom.read_file(file_name)
                self.poutput(p)
            else:
                self.poutput('Could not find file {}'.format(file_name))
        elif count == 1:
            p = pydicom.read_file(counted_files[0])
            self.poutput(p)
        elif count > 1:
            self.poutput('Found multiple files with same name:')
            for f in counted_files:
                self.poutput(f)
            self.poutput('Please select full file path of file you want and repeat this command')

    def do_check_pixels(self, verbose):
        """
        Usage: check_pixels [verbose]
        Check that each DICOM file contains pixel values that can be loaded using pydicom and NumPy. Sometimes,
        images may be compressed in some way (e.g., using JPEG2000). In that case, they cannot be loaded with
        pydicom and their pixel values need to be extracted to raw format.
        """
        files = self.result_manager.get_current_result_data()
        count = 0
        for f in files:
            p = pydicom.read_file(f)
            try:
                p.convert_pixel_data()
                if verbose != '':
                    self.poutput('OK: {}'.format(f))
            except NotImplementedError:
                count += 1
                self.poutput('ERROR: could not load pixel data {}'.format(f))
        if count > 0:
            self.poutput('Pixel data for {} out of {} files could not be loaded'.format(count, len(files)))
        else:
            self.poutput('Pixel data OK for all files')

    def do_dump_scan_props(self, output_file):
        """
        Usage: dump_scan_props output_file
        Dumps following scanning properties of each DICOM file:

         - Manufacturer (0x8, 0x70)
         - Manufacturer's Model Name (0x8, 0x1090)
         - Software Version(s) (0x18, 0x1020)
         - Exposure Time (0x18, 0x1150)
         - X-Ray Tube Current (0x18, 0x1151)
         - KVP (0x18, 0x60)
         - Slice Thickness (0x18, 0x50)
         - Pixel Spacing (0x28, 0x30)
         - Convolution Kernel (0x18, 0x1210)
         - Filter Type (0x18, 0x1160)
         - Patient Position (0x18, 0x5100)
         - Institution Name (0x8, 0x80)
         - Requested Procedure Description (0x32, 0x1060)

        The output is written to a CSV file for easy analysis.
        """
        files = self.result_manager.get_current_result_data()
        rows = []
        for file_path in files:

            p = pydicom.read_file(file_path)

            manufacturer = ''
            manufacturer_tag = (0x8, 0x70)
            if manufacturer_tag in p:
                manufacturer = p[manufacturer_tag].value

            model_name = ''
            model_name_tag = (0x8, 0x1090)
            if model_name_tag in p:
                model_name = p[model_name_tag].value

            software_version = ''
            software_version_tag = (0x18, 0x1020)
            if software_version_tag in p:
                software_version = p[software_version_tag].value

            exposure_time = ''
            exposure_time_tag = (0x18, 0x1150)
            if exposure_time_tag in p:
                exposure_time = p[exposure_time_tag].value

            tube_current = ''
            tube_current_tag = (0x18, 0x1151)
            if tube_current_tag in p:
                tube_current = p[tube_current_tag].value

            kvp = ''
            kvp_tag = (0x18, 0x60)
            if kvp_tag in p:
                kvp = p[kvp_tag].value

            slice_thickness = ''
            slice_thickness_tag = (0x18, 0x50)
            if slice_thickness_tag in p:
                slice_thickness = p[slice_thickness_tag].value

            pixel_spacing = ''
            pixel_spacing_tag = (0x28, 0x30)
            if pixel_spacing_tag in p:
                pixel_spacing = p[pixel_spacing_tag].value

            convolution_kernel = ''
            convolution_kernel_tag = (0x18, 0x1210)
            if convolution_kernel_tag in p:
                convolution_kernel = p[convolution_kernel_tag].value

            filter_type = ''
            filter_type_tag = (0x18, 0x1160)
            if filter_type_tag in p:
                filter_type = p[filter_type_tag].value

            patient_position = ''
            patient_position_tag = (0x18, 0x5100)
            if patient_position_tag in p:
                patient_position = p[patient_position_tag].value

            institute_name = get_institute_name(file_path)

            requested_procedure = ''
            requested_procedure_tag = (0x32, 0x1060)
            if requested_procedure_tag in p:
                requested_procedure = p[requested_procedure_tag].value

            line = '{};{};{};{};{};{};{};{};{};{};{};{};{};{}'.format(
                file_path, manufacturer, model_name, software_version, exposure_time, tube_current, kvp,
                slice_thickness, pixel_spacing, convolution_kernel, filter_type, patient_position, institute_name,
                requested_procedure)
            rows.append(line)
            self.poutput(line)

        header = 'file_path;manufacturer;model_name;software_version;exposure_time;tube_current;kvp;slice_thickness;' \
                 'pixel_spacing;convolution_kernel;filter_type;patient_position;institute_name;requested_procedure'
        self.poutput('Writing scan properties to {}'.format(output_file))
        with open(output_file, 'w') as f:
            f.write(header + '\n')
            for row in rows:
                f.write(row + '\n')
        self.poutput('Finished writing')


def main():
    import sys
    shell = DicomExplorerShell()
    sys.exit(shell.cmdloop())


if __name__ == '__main__':
    main()



import os
import argparse
import h5py
import pydicom

from barbell2.lib.dicom import Dcm2Numpy, Tag2NumPy


class MyException(Exception):
    pass


def info_requested(args):
    if len(args) == 2 and args[1] == '--info':
        return True
    return False


def show_info():
    print("""The createh5.py script allows you to create HDF5 files for training, validation and testing your
    deep learning networks in Keras.""")


def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('root_dir',
                        help='Root folder containing collection folders')
    parser.add_argument('output_dir',
                        help='Output folder where train.h5, validation.h5 and test.h5 will be written')
    parser.add_argument('--delete_output_dir', type=int,
                        help='Whether to delete the output directory first (default: 0)', default=0)
    parser.add_argument('--output_file_name_training',
                        help='Output file name training set', default='training.h5')
    parser.add_argument('--output_file_name_validation',
                        help='Output file name validation set', default='validation.h5')
    parser.add_argument('--output_file_name_testing',
                        help='Output file name test set', default='testing.h5')
    parser.add_argument('--height', type=int,
                        help='Image height in pixels (rows in DICOM header)', default=512)
    parser.add_argument('--width', type=int,
                        help='Image width in pixels (columns in DICOM header)', default=512)
    parser.add_argument('--training',
                        help='Comma-separated list of collection names for training')
    parser.add_argument('--validation',
                        help='Comma-separated list of collection names for validation')
    parser.add_argument('--testing',
                        help='Comma-separated list of collection names for validation')
    # https://stackoverflow.com/questions/8259001/python-argparse-command-line-flags-without-arguments
    # parser.add_argument('--info', help='Shows detailed help info', action='store_true')
    args = parser.parse_args()
    return args


def check_collections(root_dir, collections):
    collection_list = [x.strip() for x in collections.split(',')]
    for collection in collection_list:
        if not os.path.isdir(os.path.join(root_dir, collection)):
            raise MyException('Collection directory "{}" does not exist'.format(collection))
    print('Done')


def has_correct_dimensions(dcm_file, width, height):
    p = pydicom.read_file(dcm_file)
    if int(p.Rows) == height and int(p.Columns) == width:
        return True
    return False


def create_h5(root_dir, collections, width, height, output_file_path):
    file_list = []
    collection_list = [x.strip() for x in collections.split(',')]
    print(collection_list)
    for collection in collection_list:
        collection_dir = os.path.join(root_dir, collection)
        for root, dirs, files in os.walk(collection_dir):
            for f in files:
                if f.endswith('.dcm') and not f.startswith('._'):
                    dcm_file = os.path.join(root, f)
                    # Only allow images to be included that have the correct dimension (e.g., 512x512)
                    if has_correct_dimensions(dcm_file, width, height):
                        tag_file = os.path.join(root, f)[:-4] + '.tag'
                        if os.path.isfile(tag_file):
                            file_list.append([dcm_file, tag_file])
                            print('Adding {} to collection'.format(dcm_file))
    print('Creating H5...')
    with h5py.File(output_file_path, 'w') as h5f:
        count = 1
        for file_pair in file_list:
            dcm_pixels = get_dcm_pixels(file_pair[0])
            tag_pixels = get_tag_pixels(file_pair[1], dcm_pixels.shape)
            group = h5f.create_group('{:04d}'.format(count))
            group.create_dataset('images', data=dcm_pixels)
            group.create_dataset('labels', data=tag_pixels)
            print('{:04d} added {}'.format(count, file_pair[0]))
            count += 1
    print('Done')


def get_dcm_pixels(file_path):
    dcm2numpy = Dcm2Numpy()
    dcm2numpy.set_input_dicom_file_path(file_path)
    dcm2numpy.execute()
    return dcm2numpy.get_output_numpy_array()


def get_tag_pixels(file_path, shape):
    tag2numpy = Tag2NumPy(shape)
    tag2numpy.set_input_tag_file_path(file_path)
    tag2numpy.execute()
    return tag2numpy.get_output_numpy_array()


def run():

    show_info()
    args = get_args()

    # Verify that root folder exists and is not empty
    if not os.path.isdir(args.root_dir):
        raise MyException('Root directory "{}" does not exist'.format(args.root_dir))
    if len(os.listdir(args.root_dir)) == 0:
        raise MyException('Root directory "{}" is empty'.format(args.root_dir))

    # Verify that output folder does not exist or delete it if so configured
    if os.path.isdir(args.output_dir):
        if args.delete_output_dir is not None and args.delete_output_dir == 1:
            os.removedirs(args.output_dir)
        else:
            raise MyException(
                'Output directory "{}" exists. Please delete it or set delete_output_dir=1'.format(args.output_dir))

    # Verify that training, validation and test collections exist and are not empty
    if args.training is not None:
        print('Checking collections training...')
        check_collections(args.root_dir, args.training)
    if args.validation is not None:
        check_collections(args.root_dir, args.validation)
    if args.testing is not None:
        check_collections(args.root_dir, args.testing)

    os.makedirs(args.output_dir, exist_ok=True)

    # Create training H5
    if args.training is not None:
        print('Creating H5 training...')
        create_h5(
            args.root_dir,
            args.training,
            args.width, args.height,
            os.path.join(args.output_dir, args.output_file_name_training))

    # Create validation H5
    if args.validation is not None:
        create_h5(
            args.root_dir,
            args.validation,
            args.width, args.height,
            os.path.join(args.output_dir, args.output_file_name_validation))

    # Create test H5
    if args.testing is not None:
        create_h5(
            args.root_dir,
            args.testing,
            args.width, args.height,
            os.path.join(args.output_dir, args.output_file_name_testing))


def main():
    try:
        run()
    except MyException as e:
        print(e)


if __name__ == '__main__':
    main()



import os
import xnat
import random
import string


class XnatClient(object):

    def __init__(self, host, user, password, project_id, test_run=True):
        if not host.startswith('http'):
            host = 'https://' + host
        self._session = xnat.connect(host, user=user, password=password)
        self._project = self._session.projects[project_id]
        self._test_run = test_run
        print('Loaded project {} containing {} subjects'.format(self._project.id, len(self._project.subjects)))

    def project(self):
        return self._project

    def create_project(self, project_id, project_name):
        project = None
        if project_id not in self._session.projects.keys():
            if self._test_run:
                print('Creating project {}'.format(project_id))
            else:
                project = self._session.classes.ProjectData(parent=self._session, label=project_id, name=project_name)
        else:
            print('Retrieving project {}'.format(project_id))
            project = self._session.projects[project_id]
        return project

    def subjects(self):
        return self._project.subjects.data.values()

    def subject(self, subject_id):
        return self._project.subjects[subject_id]

    def create_subject(self, subject_id):
        subject = None
        if subject_id not in self._project.subjects.keys():
            if self._test_run:
                print('Creating subject {}'.format(subject_id))
            else:
                subject = self._session.classes.SubjectData(parent=self._project, label=subject_id)
        else:
            print('Retrieving subject {}'.format(subject_id))
            subject = self._project.subjects[subject_id]
        return subject

    def delete_project(self, project_id):
        project = self._session.projects[project_id]
        self.delete_all_subjects(project)
        project.delete()

    def delete_subject(self, subject, verbose=True):
        subject_id = subject.label
        if self._test_run and verbose:
            print('Deleting subject {}'.format(subject.label))
        else:
            experiments = subject.experiments.data.values()
            for experiment in experiments:
                experiment.delete()
            subject.delete()
            if verbose:
                print('Deleted subject {}'.format(subject_id))

    def delete_all_subjects(self, project):
        subjects = project.subjects.data.values()
        count = 0
        for subject in subjects:
            self.delete_subject(subject, verbose=False)
            print('Deleted subject {}/{}'.format(count, len(subjects)))
            count += 1

    def create_experiment(self, experiment_id, subject):
        # Experiment ID must be unique within whole project so prepend with subject ID
        experiment_id = '{}_{}'.format(subject.label, experiment_id)
        experiment = None
        if self._test_run:
            print('Creating experiment {}'.format(experiment_id))
        else:
            if experiment_id not in subject.experiments.keys():
                experiment = self._session.classes.CtSessionData(parent=subject, label=experiment_id)
            else:
                print('Retrieving experiment {}'.format(experiment_id))
                experiment = subject.experiments[experiment_id]
        return experiment

    def create_resource_folder(self, resource_folder_id, experiment):
        # Resource folder ID needs to be unique only within experiment. Within this resource
        # folder you can upload multiple resource files.
        resource = None
        if resource_folder_id not in experiment.resources.keys():
            if self._test_run:
                print('Creating resource folder {}'.format(resource_folder_id))
            else:
                resource = self._session.classes.ResourceCatalog(parent=experiment, label=resource_folder_id)
        else:
            print('Retrieving resource folder {}'.format(resource_folder_id))
            resource = experiment.resources[resource_folder_id]
        return resource

    def upload_resource(self, file_path, subject_id, experiment_id, resource_folder_id, overwrite=True):
        subject = self.subject(subject_id)
        experiment = subject.experiments[experiment_id]
        resource = self.create_resource_folder(resource_folder_id, experiment)
        if self._test_run:
            print('Uploading resource {} to resource folder {} of experiment {} for subject {} in project {}'.format(
                file_path,
                resource_folder_id,
                experiment_id,
                subject_id, self._project.id))
        else:
            resource.upload(file_path, os.path.split(file_path)[1], overwrite=overwrite)
            print('Uploaded resource {} to resource folder {} of experiment {} for subject {} in project {}'.format(
                file_path,
                resource_folder_id,
                experiment_id,
                subject_id, self._project.id))

    def share_experiments(self, experiment_ids, target_project_id):
        if ' ' in target_project_id:
            raise RuntimeError('Target project ID cannot contain spaces!')
        # Get target project object
        random_id = ''.join(random.choices(string.ascii_uppercase + string.digits, k=16))
        target_project_id = '{}_{}'.format(target_project_id, random_id)
        target_project = self.create_project(target_project_id, target_project_id)
        # Run through list of experiment IDs
        for experiment_id in experiment_ids:
            # Get subject ID from experiment ID which has format <subjectId>_<experimentName>
            subject_id = experiment_id.split('_')[0]
            # Share subject. You must do this otherwise the shared experiment will not be visible
            # in the target project
            if subject_id not in self._project.subjects.keys():
                raise RuntimeError('Subject {} does not exist in source project'.format(subject_id))
            subject = self.subject(subject_id)
            if self._test_run:
                print('Sharing subject {} with target project {}'.format(subject_id, target_project_id))
                print('Sharing subject {} experiments {}'.format(subject_id, experiment_id))
            else:
                # If the subject has already been shared with the target project, skip this step
                if subject_id in target_project.subjects.keys():
                    print('Subject {} has already been shared with target project'.format(subject_id))
                else:
                    subject.share(target_project_id, label=subject_id)
                    print('Shared subject {} with project {}'.format(subject_id, target_project_id))
                # Share experiment
                if experiment_id not in subject.experiments.keys():
                    raise RuntimeError('Experiment {} does not exist in source project'.format(experiment_id))
                experiment = subject.experiments[experiment_id]
                # If the experiment has already been shared with the target project, skip this step
                if experiment_id in target_project.experiments.keys():
                    print('Experiment {} has already been shared with target project'.format(experiment_id))
                else:
                    experiment.share(target_project_id, label=experiment_id)
                    print('Shared experiment {} of subject {} with project {}'.format(experiment_id, subject_id, target_project_id))
        return target_project_id
